#include "Power.h"
#include <WiFi.h>              // Built-in

unsigned long batteryTimer = 0;           // Timer for low battery measure
boolean powerSwitchWasReleased = false;   // Power switch has been released since power on

void Power_init() {
  //
  // pin to measure battery level
  pinMode(PIN_BATTERY, INPUT);  
  // pin to control power
  pinMode(PIN_POWER_CONTROL, OUTPUT);
  // pin to check power switch
  pinMode(PIN_POWER_SWITCH, INPUT);  
  // Latch power on
  Power_latchOn();
  // Wifi and bluetooth off to save power
  WiFi.mode(WIFI_OFF);
  btStop();

  

}

void Power_latchOn() {
  //
  // Switch on the peripherals 
  //
  Serial.println("POWER - Power on");
  digitalWrite(PIN_POWER_CONTROL, HIGH);

}

boolean Power_offRequested() {
  //
  // Returns true if the power switch has been pushed again, requesting the system to shut down. We detect here
  // the change from LOW to HIGH of the power switch. 
  //
  if (digitalRead(PIN_POWER_SWITCH) == LOW) {
    powerSwitchWasReleased = true;       // Memorize that the switch has been released since power on
    return false;
  } else {
    // Power switch activated (HIGH), wait a bit: anti-bounce detection
    delay(200);
    if (digitalRead(PIN_POWER_SWITCH) == HIGH) {
      // Power switch is HIGH (pushed) and it was previously released: power off requested
      if (powerSwitchWasReleased) {
        // Wait until it is released
        while (digitalRead(PIN_POWER_SWITCH) == HIGH)
          delay(200);
        return true;
      }
      // Power switch is HIGH (pushed) but never released since power on: it's just an early measure after the boot
      else 
        return false;
    } else
      // False detection, power switch activation too short
      return false;
  }
}

float Power_readBatteryValue() {
  //
  // Read voltage level from divider bridge 1.5Mohms / 470Kohms, with a correction factor determined by experience
  // Reference value is 3.3v
  //
  const int N = 10;
  // The value is computed as the mean of N samples
  float voltage = 0;
  for (int i=1; i<=N; i++) {
    voltage += analogRead(PIN_BATTERY) / 4096.0 * 3.3 * (1.5+0.47)/1.5;   //  value is 4096 for 3,3 volts
  }
  voltage = voltage/N;
  Serial.println("Voltage = " + String(voltage));
  return voltage;
}

uint8_t Power_getBatteryLevel() {
  //
  // returns percentage 0..100 of remaining charge, Power_readBatteryValue() must be called first to refresh the voltage value
  //
  uint8_t percentage;

  float voltage = Power_readBatteryValue();
  if (voltage >= VMAX) percentage = 100;
  else if (voltage <= VMIN) percentage = 0;
  else percentage = round((voltage - VMIN) / (VMAX - VMIN) * 100.0);
  Serial.println("Battery level = " + String(percentage) + "%");  
  return percentage;
}
/*
boolean Power_batteryIsLow() {
  //
  // Returns true if battery level has been below the VMIN value for a delay > Power_LOW_WARNING_DELAY
  //
  Power_readBatteryValue();
  if (Power_getBatteryLevel() == 0) {
    // Need a few iterations to confirm the battery is empty
    if (batteryTimer > 0) {
      // count down has already started,limit reached ?
      if ((millis() - batteryTimer) > POWER_LOW_WARNING_DELAY) {
        return true;
      } else {
        return false;
      }   
    } else {
      // cound down is triggered, start the timer
      batteryTimer = millis();
      return false;
    }
  } else {
    // Battery is still ok
    batteryTimer = 0;
    return true;
  }
}

void Power_goDeepSleep() {
  //
  //Go to sleep now
  //
  Serial.println("Going to sleep now");
  //
  delay(1000);
  esp_deep_sleep_start();

}
*/
void Power_shutDown() {

  Serial.println("Shutting down");
  digitalWrite(PIN_POWER_CONTROL, LOW);
  
}
/*
void Power_printWakeupReason() {
  
  esp_sleep_wakeup_cause_t wakeup_reason;

  wakeup_reason = esp_sleep_get_wakeup_cause();

  switch(wakeup_reason)
  {
    case ESP_SLEEP_WAKEUP_EXT0 : Serial.println("Wakeup caused by external signal using RTC_IO"); break;
    case ESP_SLEEP_WAKEUP_EXT1 : Serial.println("Wakeup caused by external signal using RTC_CNTL"); break;
    case ESP_SLEEP_WAKEUP_TIMER : Serial.println("Wakeup caused by timer"); break;
    case ESP_SLEEP_WAKEUP_TOUCHPAD : Serial.println("Wakeup caused by touchpad"); break;
    case ESP_SLEEP_WAKEUP_ULP : Serial.println("Wakeup caused by ULP program"); break;
    default : Serial.printf("Wakeup was not caused by deep sleep: %d\n",wakeup_reason); break;
  }
}
*/
